from sensor.Sensor import Sensor
from utils.datatypes import TYPE_INT
from xml.sax.saxutils import escape
import math
import os

import calls

EXIM = "/usr/sbin/exim"
SPOOL_DIR = "/var/spool/exim"
LOG_FILE = "/var/log/exim/mainlog"
 
class EximonSensor(Sensor):

    colors = {'frozen': '#FF0000',
              '':       '#FFFFFF'}
    
    def __init__(self, *args):
        Sensor.__init__(self)

        self._set_config_type("queue_len", TYPE_INT, 8)
        self._set_config_type("log_len", TYPE_INT, 12)
        self._set_config_type("plot_interval", TYPE_INT, 30)

        self._counter = 0
        self._log = []
        self._logfile = open(LOG_FILE, 'r')
        self._logfile.read()
        self._add_timer(2000, self._update)
        self._mouseOverMsg = None
        self._showInfo = False
        self._plotData = {'in':0, 'out':0, 'local':0, 'smtp':0}

    def _update(self):

        queue_dlen = self._get_config("queue_len")
        log_dlen = self._get_config("log_len")

        output = self._new_output()
        
        queue = self._queue = os.listdir(SPOOL_DIR + "/msglog")

        newlog = self._logfile.readlines()
        self.trackLog(newlog)
        self._log = (self._log + newlog)[-log_dlen:]

        # update plots
        if self._counter == 0:
            self._counter = self._get_config("plot_interval")

            # multiply by 10 for nicer scaling
            output.set("queuePlot", len(queue)*10)
            output.set("queuePlotCaption", 'Queue: '+ str(len(queue)))
            output.set("inPlot", self._plotData["in"]*10)
            output.set("inPlotCaption", 'In: '+ str(self._plotData["in"]))
            output.set("outPlot", self._plotData["out"]*10)
            output.set("outPlotCaption", 'Out: '+ str(self._plotData["out"]))
            output.set("localPlot", self._plotData["local"]*10)
            output.set("localPlotCaption", 'Local: '+ str(self._plotData["local"]))
            output.set("smtpPlot", self._plotData["smtp"]*10)
            output.set("smtpPlotCaption", 'SMTP: '+ str(self._plotData["smtp"]))
            self._plotData = {'in':0, 'out':0, 'local':0, 'smtp':0}
        else:
            self._counter -= 1

        if self._showInfo:
            queue = queue[-queue_dlen:]
            output.set("queueLen", len(queue))
            if self._mouseOverMsg is None:
                log = self.getGlobalLog()
                output.set("logtail", log)

            for index, id in enumerate(queue):
                status = self.getStatus(id)
                subject = self.getHeader(id, 'Subject:')
                date = self.getHeader(id, 'Date:')
                if subject and date:
                    title = '"%s", %s' % (subject, date)
                else:
                    title = 'Message ID: ' + id
                    
                color = self.colors[status]
                output.set("queue[%d]" % index, title)
                output.set("queueColor[%d]" % index, color)

        self._send_output(output)

        return 1

    def trackLog(self, entries):
        for entry in entries:
            if entry.find('=>') != -1:
                self._plotData['out'] += 1
                if entry.find('D='):
                    self._plotData['local'] += 1
                elif entry.find('T=remote_smtp'):
                    self._plotData['smtp'] += 1
            elif entry.find('<=') != -1:
                self._plotData['in'] += 1

    def getMsgLog(self, id):
        log_dlen = self._get_config("log_len")
        f = os.popen(EXIM + ' -Mvl ' + id, 'r')
        log = f.readlines()
        f.close()
        log = self.wrapText(log, margin=60, indent=8)
        log = escape("".join(log[-log_dlen:]).strip())
        return log

    def getGlobalLog(self):
        log_dlen = self._get_config("log_len")
        log = self.wrapText(self._log, margin=60, indent=8)
        log = "".join(log[-log_dlen:]).strip()
        log = escape(log)
        return log

    def getStatus(self, id):
        log = self.getMsgLog(id)
        if len(log) == 0: return ''
        log_entry = log.splitlines()[-1]
        if log_entry.count('*** Frozen') == 1:
            return 'frozen'
        else:
            return ''

    def getHeader(self, id, header):
        f = os.popen(EXIM + ' -Mvh ' + id, 'r')
        headers = f.readlines()
        f.close()
        header = filter(lambda s: s.startswith(header), headers)
        if len(header) > 0:
            return header[0][len(header):].strip()
        else:
            return ''

    def getMsgMenu(self, id):
        # format: [(label, active, [submenus], callback, [args]), ...]
        status = self.getStatus(id)
        menu = [
            ("Freeze message", status != 'frozen', None, calls.freezeMessage, [id]),
            ("Thaw message", status == 'frozen', None, calls.thawMessage, [id]),
            ("Delete message", True, None, calls.deleteMessage, [id]),
            ("Edit message", True, None, calls.editMessage, [id]),
            ("Give up on message", True, None, calls.giveupMessage, [id])
            ]
        return menu

    def getLogMenu(self):
        menu = [
            ("Run queue now", True, None, calls.sendReady, []),
            ("Send all unfrozen", True, None, calls.sendNotFrozen, []),
            ("Send all", True, None, calls.sendAll, [])
            ]
        return menu

    def wrapText(self, line_list, margin, indent):
        result = []
        for l in line_list:
            nl = l
            while len(nl) > margin:
                result.append(nl[:margin]+"\n")
                nl = (" " * indent) + nl[margin:]
            if nl.strip():
                result.append(nl)
        return result
 
    def call_action(self, action, path, args=[]):
        if path and len(self._queue) >= path[0]+1:
            active_id = self._queue[path[0]]
        if action == "msgMenu":
            menu = self.getMsgMenu(active_id)
            self._open_menu(menu)
        elif action == "logMenu":
            menu = self.getLogMenu()
            self._open_menu(menu)
        elif action == "enterMsg":
            self._mouseOverMsg = path[0]
            output = self._new_output()
            msglog = self.getMsgLog(active_id)
            output.set('logtail', msglog)
            self._send_output(output)
        elif action == "leaveMsg":
            if self._mouseOverMsg == path[0]:
                self._mouseOverMsg = None
                output = self._new_output()
                logtail = self.getGlobalLog()
                output.set('logtail', logtail)
                self._send_output(output)
        elif action == "clickPlot":
            self._showInfo = not self._showInfo
            if not self._showInfo:
                output = self._new_output()
                output.set('logtail', '')
                output.set('queueLen', 0)
                self._send_output(output)
        else:
            print "Unknown action:", action, path, args

    def get_configurator(self):
        configurator = self._new_configurator()
        configurator.set_name("Eximon desklet configuration")

        configurator.add_title("Plots")
        configurator.add_spin("Interval:", "plot_interval", "", 1, 3600)

        configurator.add_title("Display limits")
        configurator.add_spin("Queue:", "queue_len", "", 1, 20)
        configurator.add_spin("Log:", "log_len", "", 1, 20)

        return configurator
    

def new_sensor(args):
    return apply(EximonSensor, args)
